/*--- (c) 2002 iWombat.com All rights reserved ---*/

package com.iwombat.foundation.environment;

import java.util.*;
import java.net.*;
import java.beans.*;
import java.io.*;



/**
 *
 * Base class for reading/writing and getting configs
 *
 * @author  $Author: bob $
 * @version $Revision: 1.9 $
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2003/09/22 20:51:52 $</dd></dl>
 */
public class ConfigurationMap{
    /**
     */
    private String name;
    private HashMap configMap = new HashMap();
    
    // useful for seeding new maps
    public ConfigurationMap() {
    }
    
    public ConfigurationMap(String name, URL location) {
        setName(name);
        read(location);
    }
    
    public void setName(String newName) {
        this.name = newName;
    }
    
    public String getName() {
        return this.name;
    }
    
    
    
    public Configuration getConfiguration(String name){
        return (Configuration)configMap.get(name);
    }
    
    public void setConfiguration(Configuration newConfig) {
        configMap.put(newConfig.getKey(), newConfig);
    }
    
    public void setMap(HashMap map) {
        configMap = map;
    }
    
    public HashMap getMap() {
        return this.configMap;
    }
    
    private void read(URL location) {
        // read the hash maps off of disk in serialized form
        XMLDecoder decoder = null;
        configMap = new HashMap();
        String path = location.getPath().substring(2);
    
        try {
            URLConnection xmlConnect = location.openConnection();
            decoder = new XMLDecoder(xmlConnect.getInputStream());
            
            // keep grabbing objects until ArrayIndexOutOfBounds creeps our way
            while (true) {
                Configuration conf = (Configuration) decoder.readObject();
                configMap.put(conf.getKey(), conf);
            }
        } catch (java.lang.ArrayIndexOutOfBoundsException e) {
            // we'll get this if there are no more objects - this is okay
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException("Unable to locate URL:" + e.getMessage());
        } catch (IOException e) {
            throw new IllegalArgumentException("Unable to read config URL:" + e.getMessage());
        } catch (RuntimeException e) {
            throw new EnvironmentException("Unable to decode config URL:" + location.toString()
                         + "  Check valid bean signatures.   " + "("+e.getClass().getName()+") "+ e.getMessage());
        } catch (Exception e) {
            throw new EnvironmentException("Unable to decode config URL:" + location.toString() 
                        + "  Check XML syntax.   " +  e.getMessage());
        } finally {
            if (null != decoder) {
                decoder.close();
            }
        }
    }
    
    public void write(String outFile) {
        XMLEncoder encoder = null;
        try {
            encoder = new XMLEncoder( new BufferedOutputStream(new FileOutputStream(outFile)));
        } catch (Exception exe) {
            throw new ConfigurationException("Can not write config to file:" + outFile + exe.getMessage(), exe);
        }
        
        Set keys = configMap.keySet();
        Iterator iter = keys.iterator();
        
        try {
            while (iter.hasNext()) {
                Configuration conf = (Configuration)configMap.get(iter.next());
                encoder.writeObject(conf);
            }
        } finally {
	    if (null != encoder) {
               encoder.close();
            }
        }
    }
    
}
