// (c) Copyright 2001-2002 iwombat.com all rights reserved
package com.iwombat.foundation.persist;

import java.sql.Connection;

/**
 * Sql Persistence provider. Facilitates the generation of sql connection objects
 * for the implemented Rdbms technology.
 *
 * @author  $Author: brentp $
 * @version $Revision: 1.2 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/11/18 16:04:40 $</dd></dl>
 */

public interface SqlPersistence extends Persistence{

        
    /**
     * retrieves an available connection
     *
     * @return the active connection object
     */
    public Connection getConnection();
    
    /**
     * resets and re-initializes the connection pool
     *
     */
    public void resetConnectionPool();
    
    /**
     * sets the database name to use
     *
     * @param the database name
     *
     */
    public void setDatabaseName(String databaseName);
    
    /**
     * retrieves the currently set database name
     *
     * @return the database name
     */
    public String getDatabaseName();
        
        
    /**
     * Sets the database server's hostname
     *
     * @param the host name to set
     */
    public void setHostname(String hostname);
    
    /**
     * retrieves the currently set hostname
     * 
     * @return the host name
     */
    public String getHostname();

    /**
     * sets the database connection password
     *
     * @param the password to set
     */
    public void setPassword(String password);
    
    /**
     * retrieves the currently set password
     *
     * @return the current password
     */
    public String getPassword();
    
    /**
     * sets the database connection user Id
     *
     * @param the user Id to set
     */
    public void setUserId(String userId);
    
    /**
     * returns the currently set user Id
     *
     * @return the currently configured user Id
     */
    public String getUserId();
    
    /**
     * sets the initial connection pool size
     *
     * @param the pool size in number of connections
     */
    public void setInitialPoolSize(int initialPoolSize);
    
    /**
     * retrieves the currently set initial pool size
     *
     * @return the initial pool size
     *
     */
    public int getInitialPoolSize();
    
    /**
     * set the maximum connection pool size
     *
     * @param the maximum number of connections to allow
     */
    public void setMaxPoolSize(int maxPoolSize);
    
    /**
     * retrieves the currently set maximum pool size
     *
     * @return the maximum pool size
     */
    public int getMaxPoolSize();

    /**
     * sets the minimum connection pool size
     *
     * @param the minimum pool size
     *
     */
    public void setMinPoolSize(int minPoolSize);
    
    /**
     * retrieves the currently set minimum pool size
     *
     * @return the minimum pool size
     */
    public int getMinPoolSize();
    
    /**
     * Returns this Persistence's configuration key name
     *
     * @return The configuration key name for this Persistence
     */
    public String getKey();
    
    /**
     * Sets the configuration key name of this Persistence
     *
     * @param the key name to set for this Persistence
     */
    public void setKey(String key);
    
    /**
     * Returns the free-form Persistence description
     *
     * @return the current Persistence Description
     */
    public String getDescription();
    
    /**
     * Sets the free-form Persistence description
     *
     * @param the description String to set
     */
    public void setDescription(String description);
            

}
