// (c) 2003 Aegis Design - all rights reserved
package com.iwombat.swing;


import java.awt.*;
import java.awt.event.*;


import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.border.*;




/**
 * FontDialog.java
 *
 * @author  $Author: bob $
 * @version $Revision: 1.2 $
 *
 * <br>
 * <dl><dt><b>Date::</b><dd>$Date: 2003/10/08 05:16:47 $</dd></dl>
 */

public class FontDialog extends JDialog
{
  protected int option = JOptionPane.CLOSED_OPTION;
  protected ListChooser fontNameList;
  protected ListChooser fontSizeList;
  protected MutableAttributeSet attributes;
  protected JCheckBox boldCheck;
  protected JCheckBox italicCheck;
  protected JCheckBox underlineCheck;    
  protected JCheckBox strikeCheck;
  protected JCheckBox subscriptCheck;
  protected JCheckBox superscriptCheck;
    
  protected JComboBox colorChooser;
  protected JLabel preview;

  private static final String FONT = "Font";
  private static final String OK = "Ok";
  private static final String PREVIEW = "Preview";
    private static final String PREVIEWFONT = "Preview Font";
  private static final String EFFECTS = "Effects";
  
  private static final String NAME = "Name:";
  private static final String SIZE = "Size:";
  private static final String COLOR = "Color:";
  
  private static final String BOLD = "Bold";
  private static final String ITALIC = "Italic";
  private static final String UNDERLINE = "Underline";
  private static final String STRIKE = "Strikethrough";
  private static final String SUB = "Subscript";
  private static final String SUPER = "Superscript";

          
  public FontDialog(JFrame parent, String[] names, String[] sizes)
  {
    super(parent, FONT, true);
    this.getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));

    JPanel p = new JPanel(new GridLayout(1, 2, 10, 2));
    p.setBorder(new TitledBorder(new EtchedBorder(), FONT));
    fontNameList = new ListChooser(names, NAME);
    p.add(fontNameList);

    fontSizeList = new ListChooser(sizes, SIZE);
    p.add(fontSizeList);
    getContentPane().add(p);

    p = new JPanel(new GridLayout(2, 3, 10, 5));
    p.setBorder(new TitledBorder(new EtchedBorder(), EFFECTS));
    boldCheck = new JCheckBox(BOLD);
    p.add(boldCheck);
    italicCheck = new JCheckBox(ITALIC);
    p.add(italicCheck);
    underlineCheck = new JCheckBox(UNDERLINE);
    p.add(underlineCheck);
    strikeCheck = new JCheckBox(STRIKE);
    p.add(strikeCheck);
    subscriptCheck = new JCheckBox(SUB);
    p.add(subscriptCheck);
    superscriptCheck = new JCheckBox(SUPER);
    p.add(superscriptCheck);
    getContentPane().add(p);

    getContentPane().add(Box.createVerticalStrut(5));
    p = new JPanel();
    p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
    p.add(Box.createHorizontalStrut(10));
    p.add(new JLabel(COLOR));
    p.add(Box.createHorizontalStrut(20));
    colorChooser = new JComboBox();

    int[] values = new int[] { 0, 128, 192, 255 };
    for (int r=0; r<values.length; r++) {
      for (int g=0; g<values.length; g++) {
        for (int b=0; b<values.length; b++) {
          Color c = new Color(values[r], values[g], values[b]);
          colorChooser.addItem(c);
        }
      }
    }

    colorChooser.setRenderer(new ColorComboRenderer());
    p.add(colorChooser);
    p.add(Box.createHorizontalStrut(10));
    getContentPane().add(p);

    p = new JPanel(new BorderLayout());
    p.setBorder(new TitledBorder(new EtchedBorder(), PREVIEW));
    preview = new JLabel(PREVIEWFONT, JLabel.CENTER);
    preview.setBackground(Color.white);
    preview.setForeground(Color.black);
    preview.setOpaque(true);
    preview.setBorder(new LineBorder(Color.black));
    preview.setPreferredSize(new Dimension(120, 40));
    p.add(preview, BorderLayout.CENTER);
    getContentPane().add(p);

    p = new JPanel(new FlowLayout());
    JPanel p1 = new JPanel(new GridLayout(1, 2, 10, 2));
    JButton btOK = new JButton(OK);
    ActionListener lst = new ActionListener() { 
      public void actionPerformed(ActionEvent e) {
        option = JOptionPane.OK_OPTION;
        System.out.println("OK pressed");
        setVisible(false);
      }
    };
    btOK.addActionListener(lst);
    p1.add(btOK);

    JButton btCancel = new JButton("Cancel");
    lst = new ActionListener() { 
      public void actionPerformed(ActionEvent e) {
        option = JOptionPane.CANCEL_OPTION;
        setVisible(false);
      }
    };
    btCancel.addActionListener(lst);
    p1.add(btCancel);
    p.add(p1);
    getContentPane().add(p);

    pack();
    setResizable(false);

    ListSelectionListener lsel = new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        updatePreview();
      }
    };
    fontNameList.addListSelectionListener(lsel);
    fontSizeList.addListSelectionListener(lsel);

    lst = new ActionListener() { 
      public void actionPerformed(ActionEvent e) {
        updatePreview();
      }
    };
    boldCheck.addActionListener(lst);
    italicCheck.addActionListener(lst);
    colorChooser.addActionListener(lst);
  }

  public void setAttributes(AttributeSet a) {
    attributes = new SimpleAttributeSet(a);
    String name = StyleConstants.getFontFamily(a);
    fontNameList.setSelected(name);
    int size = StyleConstants.getFontSize(a);
    fontSizeList.setSelectedInt(size);
    boldCheck.setSelected(StyleConstants.isBold(a));
    italicCheck.setSelected(StyleConstants.isItalic(a));
    underlineCheck.setSelected(StyleConstants.isUnderline(a));
    strikeCheck.setSelected(StyleConstants.isStrikeThrough(a));
    subscriptCheck.setSelected(StyleConstants.isSubscript(a));
    superscriptCheck.setSelected(StyleConstants.isSuperscript(a));
    colorChooser.setSelectedItem(StyleConstants.getForeground(a));
    updatePreview();
  }

  public AttributeSet getAttributes() {
    if (attributes == null)
      return null;
    StyleConstants.setFontFamily(attributes, fontNameList.getSelected());
    StyleConstants.setFontSize(attributes, fontSizeList.getSelectedInt());
    StyleConstants.setBold(attributes, boldCheck.isSelected());
    StyleConstants.setItalic(attributes, italicCheck.isSelected());
    StyleConstants.setUnderline(attributes, underlineCheck.isSelected());
    StyleConstants.setStrikeThrough(attributes, strikeCheck.isSelected());
    StyleConstants.setSubscript(attributes, subscriptCheck.isSelected());
    StyleConstants.setSuperscript(attributes, superscriptCheck.isSelected());
    StyleConstants.setForeground(attributes, (Color)colorChooser.getSelectedItem());
    return attributes;
  }

  public int getOption() { return option; }

  protected void updatePreview() {
    String name = (String)fontNameList.getSelected();
    int size = fontSizeList.getSelectedInt();
    if (size <= 0)
      return;
    int style = Font.PLAIN;
    if (boldCheck.isSelected()) style |= Font.BOLD;
    if (italicCheck.isSelected()) style |= Font.ITALIC;

    // Bug Alert! This doesn't work if only style is changed.
    Font fn = new Font(name, style, size);
    preview.setFont(fn);

    Color c = (Color)colorChooser.getSelectedItem();
    preview.setForeground(c);
    preview.repaint();
  }
  
  
  public class ColorComboRenderer extends JPanel implements ListCellRenderer {
      protected Color color = Color.black;
      protected Color focusColor =  (Color) UIManager.get("List.selectionBackground");
      protected Color nonFocusColor = Color.white;
      
      public Component getListCellRendererComponent(JList list,
      Object obj, int row, boolean sel, boolean hasFocus) {
          if (hasFocus || sel)
              setBorder(new CompoundBorder(
              new MatteBorder(2, 10, 2, 10, focusColor),
              new LineBorder(Color.black)));
          else
              setBorder(new CompoundBorder(
              new MatteBorder(2, 10, 2, 10, nonFocusColor),
              new LineBorder(Color.black)));
          
          if (obj instanceof Color)
              color = (Color) obj;
          return this;
      }
      
      public void paintComponent(Graphics g) {
          setBackground(color);
          super.paintComponent(g);
      }
  }
}
