/*--- (c) 2002 iWombat.com All rights reserved ---*/

package com.iwombat.tools.unitTest;



import java.util.*;
import java.net.*;
import java.io.*;
import java.beans.*;


/**
 *
 * This utility class aids in the loading of XMLEncoded collections of domain objects
 * used in unit test cases.
 *
 * @author  $Author: bob $
 * @version $Revision: 1.7 $
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/11/21 20:59:57 $</dd></dl>
 * 
 */
public class DomainObjectLoader{

   /**
    * This method takes a class name and looks for <classname>.xml in the classpath.
    * Decodes any instances of the class found therein and returns the resulting collection
    *
    *
    * @param domainClass - The class of the domain object
    * @param config name - the name of the specific config
    */
    public static Collection getDomainObjects(Class domainClass) {
        XMLDecoder decoder = null;
        String className = domainClass.getName() + ".xml";
       	ArrayList domainList = new ArrayList();
        try {
        	InputStream in = DomainObjectLoader.class.getClassLoader().getResourceAsStream(className);
        	decoder = new XMLDecoder(in);
       
                while(true) {
                          domainList.add(decoder.readObject());
                }
        } catch (java.lang.ArrayIndexOutOfBoundsException e) {
               // we're out of objects
        } catch (RuntimeException e) {
            throw new IllegalArgumentException("Unable to find:" + className
                         + "  Check valid bean signatures.   " + "("+className+") "+ e.getMessage()); 
        } catch (Exception e) {
            throw new IllegalArgumentException("Unable to decode config URL:" + className
                        + "  Check XML syntax.   " +  e.getMessage());
        } finally {
            if (null != decoder) {
                decoder.close();
            }
        }

        return domainList;
    }


   public static void writeDomainObjects(Collection domainObjects) {
	String path = TstTimeResourceLocator.getTstTimeResourcePath();
	Iterator iter = domainObjects.iterator();
        Class objClass = iter.next().getClass();
        String fullPath = path + objClass.getName() + ".xml";
        XMLEncoder encoder = null;
	try { 
		System.out.println("Encoding: " + objClass.getName() + "  To File: " + fullPath);
		encoder = new XMLEncoder( new BufferedOutputStream(new FileOutputStream(fullPath)));
        } catch (Exception exe) {
             throw new RuntimeException("Failed to establish encoder for:" + fullPath);
        }

        iter = domainObjects.iterator();
        try {
	      System.out.println("Begin Encoding: " + objClass.getName());
              while(iter.hasNext()) {
			Object domainObj = iter.next();
			encoder.writeObject(domainObj);
		}
	      System.out.println("End Encoding: " + objClass.getName());
        } catch (Exception exe) {
             throw new RuntimeException("Failed to establish and encode objects for:" + fullPath);
	} finally {
		if (null != encoder) {
			encoder.close();
		}
	}
   }

}
