// (c) 2003 iWombat.com All rights reserved

package com.iwombat.util;

import java.io.Serializable;

/**
 * Wrapper object for binary coded decimal
 *
 * @author  $Author: bob $
 * @version $Revision: 1.2 $
 *
 * <br>
 * <dt><b>Date:</b><dd>$Date: 2003/09/04 18:37:04 $</dd>
 */
public class BinaryCodedDecimal extends Number implements Serializable {
    
    long internalLong = 0L;
    
    /** Creates a new instance of BinaryCodedDecimal */
    public BinaryCodedDecimal(byte[] bcdValue) {
        this.internalLong = toReferenceLong(bcdValue);        
    }

    public BinaryCodedDecimal(long longValue) {
        internalLong = longValue;        
    }
    
    public double doubleValue() {
        return (double)internalLong;
    }
    
    public float floatValue() {
        return (float)internalLong;
    }
    
    public int intValue() {
        return (int)internalLong;
    }
    
    public long longValue() {
        return internalLong;
    }
    
    public byte byteValue() {
        return (byte)internalLong;      
    }
    
    public byte[] toRawBCDValue(int length) {
        short nibbleMask = 0x0f;
        
        byte[] rawValue = new byte[length];
        char[] charArray = Long.toString(internalLong).toCharArray();
        
        // handle a smaller array than the length
        if (charArray.length < 2*length) {
            char[] bigArray = new char[2*length];
            for (int j= bigArray.length-1; j>=0; j--) {
                bigArray[j] = '0';
            }
            for(int i=charArray.length-1; i>=0; i--) {
                bigArray[(bigArray.length-1)-i] = charArray[i];
            }
            charArray = bigArray;
        }
        
        int charIdx = 0;
        for (int i=0; i<length; i++) {
            int value;
            value = Character.digit(charArray[charIdx++], 10);
            byte highValue = (byte)((value & nibbleMask) << 4);
            value = Character.digit(charArray[charIdx++], 10);
            byte lowValue  = (byte)(value & nibbleMask);
            rawValue[i] = (byte) (highValue | lowValue);
        }
       
        return rawValue;
    }
    
    
    //public ByteArray byteArrayValue() {
    //    return new ByteArray(internalLong);
    //}
    
    public static double toDoubleValue(byte[] bcdValue) {
        return (double)toReferenceLong(bcdValue);
    }
    
    public static float toFloatValue(byte[] bcdValue) {
        return (float)toReferenceLong(bcdValue);
    }
    
    public static int toIntValue(byte[] bcdValue) {
        return (int)toReferenceLong(bcdValue);
    }
    
    public static short toShortValue(byte bcdValue) {
       byte[] byteArray = {bcdValue};
       return (short)toReferenceLong(byteArray);
    }
    
    public static long toLongValue(byte[] bcdValue) {
        return toReferenceLong(bcdValue);
    }
    
    public static byte toByteValue(byte[] bcdValue) {
        return (byte)toReferenceLong(bcdValue);        
    }

    public static byte[] toBCDByteArrayValue(long longValue, int length) {
	BinaryCodedDecimal bcd = new BinaryCodedDecimal(longValue);
        return bcd.toRawBCDValue(length);
    }
    
 
    private static long toReferenceLong(byte[] bcdValue) {
        
        short nibbleMask = 0x0f;
        
        int length = bcdValue.length;
        long  theLong = 0L;
        
        for (int pos=0; pos < bcdValue.length; pos++) {
            byte lowNibble = (byte)(bcdValue[pos] & nibbleMask);
            byte highNibble = (byte)(bcdValue[pos] >>> 4);
            theLong *=10;
            theLong +=  highNibble & nibbleMask;
            theLong *= 10;
            theLong +=  lowNibble & nibbleMask;
        }
        
        return theLong;
        
    }
    
}
