// (c) Copyright 2001-2002 iwombat.com all rights reserved
package com.iwombat.util;

/**
 * Class ObjectUtil
 * Description: Utility methods for Objects in general
 *
 * @author  $Author: brentp $
 * @version $Revision: 1.1 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/11/18 16:04:43 $</dd></dl>
 */
import com.iwombat.foundation.environment.ConfigurationLocator;
import com.iwombat.foundation.WombatRuntimeException;

public class ObjectUtil {
    

    /**
     * Generic factory to create objects based on environment configuration parameters
     *
     * @param the configuration key mapping to the appropriate property file
     * @param the key within the specified property file that maps to the class name for which the object will be created
     */
    public static final Object createObjectFromConfig(String configKey, String propertyKey){
     
        //retrive the property value based on the config parameters
        String propertyValue=ConfigurationLocator.getPropertyValue(configKey, propertyKey);
        
        if (null==propertyValue) throw new WombatRuntimeException("Unable to retrieve property value specificed by configKey '"+configKey+"'; propertyKey '"+propertyKey+"'");
        
        Object newObject=null;
        try{
            newObject=Class.forName(propertyValue).newInstance();
        }
        catch (ClassNotFoundException e){
            throw new WombatRuntimeException("Could not find class '" +propertyValue+"' specificed by configKey '"+configKey+"'; propertyKey '"+propertyKey+"'");
        }
        catch (InstantiationException e){
            throw new WombatRuntimeException("Caught InstantiationException while creating instance of '" +propertyValue+"' specificed by configKey '"+configKey+"'; propertyKey '"+propertyKey+"'");
        }
        catch (IllegalAccessException e){
            throw new WombatRuntimeException("Caught IllegalAccessException while creating instance of '" +propertyValue+"' specificed by configKey '"+configKey+"'; propertyKey '"+propertyKey+"'");
        }

        return newObject;
        
    }
    
    /** force singleton*/
    private ObjectUtil() {
    }
    
}
