// (c) Copyright 2001-2002 iwombat.com all rights reserved

package com.iwombat.util;
/**
 * VariantEnum extends Enums to provide the various enumerated types of data that
 * are stored in a Variant objects' instance. In addition to the standard Enums
 * behavior, additional behavior provides string representation for types as
 * well as static methods that cross check integer and string datatype values.
 *
 * @author  $Author: bob $
 * @version $Revision: 1.3 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/10/01 17:54:16 $</dd></dl>
 */

public final class VariantEnum extends Enums
{
    static private final int BOOLEAN_P  = 0;
    static private final int CHAR_P     = 1;
    static private final int BYTE_P     = 2;
    static private final int SHORT_P    = 3;
    static private final int INT_P      = 4;
    static private final int LONG_P     = 5;
    static private final int FLOAT_P    = 6;
    static private final int DOUBLE_P   = 7;

    // reference types

    static private final int BYTE_ARRAY_R = 8;
    static private final int STRING_R     = 9;
    static private final int JAVA_SQL_TIMESTAMP_R = 10;
    static private final int OBJECT_R = 11;
    static private final int UNKNOWN_TYPE = 12; // toString() fails if invalid index value of -1

    static private final int[] m_enums = { BOOLEAN_P, CHAR_P, BYTE_P, SHORT_P, INT_P,
                                           LONG_P, FLOAT_P, DOUBLE_P, BYTE_ARRAY_R,
                                           STRING_R, JAVA_SQL_TIMESTAMP_R, OBJECT_R,
                                           UNKNOWN_TYPE };

    // primitive types

    static private  final String BOOLEAN_PS = "boolean";
    static private  final String CHAR_PS    = "char";
    static private  final String BYTE_PS    = "byte";
    static private  final String SHORT_PS   = "short";
    static private  final String INT_PS     = "int";
    static private  final String LONG_PS    = "long";
    static private  final String FLOAT_PS   = "float";
    static private  final String DOUBLE_PS  = "double";

    // reference types

    static private  final String BYTE_ARRAY_RS          = "byte_array";
    static private  final String STRING_RS              = "String";
    static private  final String JAVA_SQL_TIMESTAMP_RS  = "java_sql_Timestamp";
    static private  final String OBJECT_RS              = "Object";

    static private  final String UNKNOWN_TYPE_S         = "UNKNOWN";

    static private final String[] enumStrings = { BOOLEAN_PS, CHAR_PS, BYTE_PS, SHORT_PS, INT_PS,
                                                  LONG_PS, FLOAT_PS, DOUBLE_PS, BYTE_ARRAY_RS,
                                                  STRING_RS, JAVA_SQL_TIMESTAMP_RS, OBJECT_RS,
                                                  UNKNOWN_TYPE_S };

    static public final VariantEnum BOOLEAN        = new VariantEnum( BOOLEAN_P );
    static public final VariantEnum CHAR           = new VariantEnum( CHAR_P );
    static public final VariantEnum BYTE           = new VariantEnum( BYTE_P );
    static public final VariantEnum SHORT          = new VariantEnum( SHORT_P );
    static public final VariantEnum INT            = new VariantEnum( INT_P );
    static public final VariantEnum LONG           = new VariantEnum( LONG_P );
    static public final VariantEnum FLOAT          = new VariantEnum( FLOAT_P );
    static public final VariantEnum DOUBLE         = new VariantEnum( DOUBLE_P );
    static public final VariantEnum BYTE_ARRAY     = new VariantEnum( BYTE_ARRAY_R );
    static public final VariantEnum STRING         = new VariantEnum( STRING_R );
    static public final VariantEnum SQL_TIMESTAMP  = new VariantEnum( JAVA_SQL_TIMESTAMP_R );
    static public final VariantEnum OBJECT         = new VariantEnum( OBJECT_R );
    static public final VariantEnum UNKNOWN        = new VariantEnum( UNKNOWN_TYPE );

    private VariantEnum( int enum )
    {
        super( enum );
    }

    /**
     * Required method for derived classes, getValues() returns the internal
     * array each derived class contains for their enum values. If a derived
     * class does not wish to implement the value array, it can return null
     * and the isEnumValue() method will return false as a result.
     *
     * @return array of integers containing the enum values or null.
     */

    protected int[] getValues()
    {
        return m_enums;
    }

    public static int[] getEnums()
    {
        return m_enums;
    }

    public String toString()
    {
        return enumStrings[ m_enum ];
    }

    public static String typeName( int type )
    {
        switch ( type )
        {
            // primitive types
            case BOOLEAN_P:       return BOOLEAN_PS;
            case CHAR_P:          return CHAR_PS;
            case BYTE_P:          return BYTE_PS;
            case SHORT_P:         return SHORT_PS;
            case INT_P:           return INT_PS;
            case LONG_P:          return LONG_PS;
            case FLOAT_P:         return FLOAT_PS;
            case DOUBLE_P:        return DOUBLE_PS;

                // reference types
            case BYTE_ARRAY_R:    return BYTE_ARRAY_RS;
            case STRING_R:        return STRING_RS;
            case JAVA_SQL_TIMESTAMP_R:   return JAVA_SQL_TIMESTAMP_RS;
            case OBJECT_R:        return OBJECT_RS;

            default:              return UNKNOWN_TYPE_S;
        }
    }

    public static int typeOf( String typeName )
    {

        // Put more common ones first for performance reasons
        if ( typeName.equals(BOOLEAN_PS) )         return BOOLEAN_P;
        if ( typeName.equals(CHAR_PS) )            return CHAR_P;
        if ( typeName.equals(BYTE_PS) )            return BYTE_P;
        if ( typeName.equals(SHORT_PS) )           return SHORT_P;
        if ( typeName.equals(INT_PS) )             return INT_P;
        if ( typeName.equals(LONG_PS) )            return LONG_P;

        if ( typeName.equals(FLOAT_PS) )           return FLOAT_P;
        if ( typeName.equals(DOUBLE_PS) )          return DOUBLE_P;

        if ( typeName.equals(BYTE_ARRAY_RS) )      return BYTE_ARRAY_R;
        if ( typeName.equals(STRING_RS) )          return STRING_R;
        if ( typeName.equals(JAVA_SQL_TIMESTAMP_RS) )   return JAVA_SQL_TIMESTAMP_R;

        if ( typeName.equals(OBJECT_RS) )           return OBJECT_R;

        return 0;
    }

    public static int typeId( String name )
    {
        for ( int i=0; i <= m_enums.length; i++ )
            if ( name.equals( typeName( i )) )
                return i;

        return UNKNOWN_TYPE;
    }
}
