// (c) Copyright 2001-2002 iwombat.com all rights reserved


package com.iwombat.util;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/* VariantList provides storage for a list of Variant objects. This is a standard Extension over java.util.ArrayList.
 * Mechanisms are in place to only allow objects of type Variant to be added to the List.
 *
 * @author  $Author: bob $
 * @version $Revision: 1.3 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/10/01 17:54:16 $</dd></dl>
 */

public class VariantList extends ArrayList{

    /** Creates new VariantList */
    public VariantList() {
        super();
    }

   /**
     * create an VariantList using the specified collection to load from
     *
     * <code>IllegalArgumentException</code> is thrown if any items in the collection is not an instance of <code>Variant</code>.
     *
     * @param c the collection to load from.
     */    
    public VariantList(Collection c){

        super(c);

        for (Iterator i=c.iterator();i.hasNext();)
            if (!(i.next() instanceof Variant))
                throw new IllegalArgumentException("Collection must contain only types of Variant");
        
    }

    /**
     * create a new, empty VariantList with the specified initial capacity
     *
     * @param initialCapacity the initial capacity of the list.
     *
     */
    
    public VariantList(int initialCapacity){
        super(initialCapacity);
    }

    /**
     * Adding an unspecified object will always throw an <code>UnsupportedOperationException</code>
     * 
     * @param index the index to insert the new object at
     * @param o the object to insert
     *
     */
        
    public void add(int index,Object o){
        throw new UnsupportedOperationException();        
    }

    /**
     * insert an Variant object at the specified index.
     *
     * @param index the index to insert the new Variant at.
     * @param v the Variant to add.
     *
     */

    public void add(int index,Variant v){
        super.add(index,v);
    }

    /**
     * Adding an unspecified object will always throw an <code>UnsupportedOperationException</code>.
     * 
     * @param o the object to add.
     * @return true always, conforms to the List interface specification.
     *
     */
    
    public boolean add(Object o){
        throw new UnsupportedOperationException();
    }

    /**
     * Add an Variant object to the list
     *
     * @param v the Variant to add
     * @return if the list changed as a result to the call.
     *
     */
    
    public boolean add(Variant v){
        return super.add(v);
    }

    /**
     * Add all of the objects in the specified collection to the list.
     * If the collection does not contain only objects of type Variant, an <code>IllegalArgumentException</code> will be thrown.
     *
     * @param c the collection containing the items to add
     * @return if the list changed as a result to the call.
     */
    
    public boolean addAll(Collection c){
     
        for (Iterator i=c.iterator();i.hasNext();)
            if (!(i.next() instanceof Variant))
                throw new IllegalArgumentException("Collection must contain only types of Variant");
        
        return super.addAll(c);
    }

    /** 
     * Add all of the objects in the specified collection to the list, inserting at the specified index.
     * If the collection does not contain only objects of type Variant, an <code>IllegalArgumentException</code> will be thrown.
     *
     * @param index the index where to insert the new objects
     * @param c the collection containing the items to add
     * @return if the list changed as a result to the call.
     *
     */
    
    public boolean addAll(int index,Collection c){
        
        for (Iterator i=c.iterator();i.hasNext();)
            if (!(i.next() instanceof Variant))
                throw new IllegalArgumentException("Collection must contain only types of Variant");

        return super.addAll(index,c);
    }

    /**
     * test if the List contains the specified object
     *
     * @param v the Variant object to test for.
     * @return true if the specified Variant resides within the list
     */
    
    public boolean contains(Variant v){
        return super.contains(v);
    }

    /**
     * retrieve an Variant object stored in the specified index.
     * 
     * @param index the index of the Variant object to retrieve.
     * @return the Variant at the specified index
     *
     */
    
    public Variant getItem(int index){
        return (Variant)super.get(index);
    }
    
    /**
     * retrieve the index of the specified Variant
     *
     * @param v the Variant for which the index is to be retrieved
     * @return the index for the specified Variant
     *
     */
    public int indexOf(Variant v){
        return super.indexOf(v);
    }
    
    
    /**
     * remove an item from the list
     * 
     * @param index the index of the item to remove
     * @return the Variant that was removed
     *
     */
    public Variant removeItem(int index){
        return (Variant)super.remove(index);
    }
    
    /**
     * set an object at the specified index.
     * Adding an unspecified object will always throw an <code>UnsupportedOperationException</code>.
     * 
     * @param index the index of the object to replace
     * @param o the object to set
     * @return the object previously at the specified location.
     *
     */
    public Object set(int index,Object o){
     
        throw new UnsupportedOperationException();
    }

    /**
     * set an Variant at the specified index.
     *
     * @param index the index of the Variant to replace
     * @param v the Variant to set
     * @return the Variant previously at the specified location.
     *
     */
    public Variant set(int index,Variant v){
        return (Variant)super.set(index,v);
    }
}
