package com.iwombat.foundation.persist;

import junit.framework.*;

import java.util.*;
import com.iwombat.util.*;
import com.iwombat.foundation.*;

/**
 * Some simple tests.
 *
 */
public class ComprehensiveDomainObjectDAOTest extends TestCase {
    
    
    public ComprehensiveDomainObjectDAOTest(String name) {
        super(name);
    }
    
    public void setUp() {
        
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(ComprehensiveDomainObjectDAOTest.class);
        return suite;
    }
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }

    public void testCreate(){
    
        ComprehensiveDomainObject cdo=new ComprehensiveDomainObjectImpl();
        
        cdo.setIdentifierValue(IdentifierFactory.createUniqueIdentifier());
        cdo.setFloatValue(new Float(1.23));
        cdo.setLongValue(new Long(333));
        cdo.setStringValue("string");
        cdo.setByteArrayValue(new ByteArray(new byte[]{0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09}));
        cdo.setBooleanValue(new Boolean(true));
        cdo.setDateValue(new Date());

        ComprehensiveDomainObjectDAO cdop=(ComprehensiveDomainObjectDAO)DAOFactory.createDAOInstance(ComprehensiveDomainObject.class);

        
        cdop.addComprehensiveDomainObject(cdo);
        ComprehensiveDomainObject cdo2;
        
        cdo2=cdop.getComprehensiveDomainObjectById(cdo.getId());
        
        assertEquals(cdo.getId(),cdo2.getId());
        assertEquals(cdo.getIdentifierValue(),cdo2.getIdentifierValue());
        assertEquals(cdo.getFloatValue(),cdo2.getFloatValue());
        assertEquals(cdo.getLongValue(),cdo2.getLongValue());
        assertEquals(cdo.getStringValue(),cdo2.getStringValue());
        assertEquals(cdo.getByteArrayValue(),cdo2.getByteArrayValue());
        assertEquals(cdo.getBooleanValue(),cdo2.getBooleanValue());
        
        //are we within a second (account for conversion imprecise-ness)
        assertTrue(Math.abs(cdo.getDateValue().getTime()-cdo2.getDateValue().getTime())<=1000);
        
        
    }
    
    public void testCreateNulls(){
        
        ComprehensiveDomainObject cdo=new ComprehensiveDomainObjectImpl();
        
        ComprehensiveDomainObjectDAO cdop=(ComprehensiveDomainObjectDAO)DAOFactory.createDAOInstance(ComprehensiveDomainObject.class);
        
        //cdo.setStringValue("blah");
        
        cdop.addComprehensiveDomainObject(cdo);
        ComprehensiveDomainObject cdo2;
        
        cdo2=cdop.getComprehensiveDomainObjectById(cdo.getId());

        assertEquals(cdo.getId(),cdo2.getId());
        assertNull(cdo2.getIdentifierValue());
        assertNull(cdo2.getFloatValue());
        assertNull(cdo2.getLongValue());
        assertNull(cdo2.getStringValue());
        assertNull(cdo2.getByteArrayValue());
        assertNull(cdo2.getBooleanValue());
        assertNull(cdo2.getDateValue());
        
    }
    

    public void testUpdate(){
        
        ComprehensiveDomainObject cdo=new ComprehensiveDomainObjectImpl();
        
        cdo.setIdentifierValue(IdentifierFactory.createUniqueIdentifier());
        cdo.setFloatValue(new Float(1.23));
        cdo.setLongValue(new Long(333));
        cdo.setStringValue("string");
        cdo.setByteArrayValue(new ByteArray(new byte[]{0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09}));
        cdo.setBooleanValue(new Boolean(true));
        cdo.setDateValue(new Date());

        ComprehensiveDomainObjectDAO cdop=(ComprehensiveDomainObjectDAO)DAOFactory.createDAOInstance(ComprehensiveDomainObject.class);
        
        cdop.addComprehensiveDomainObject(cdo);
        ComprehensiveDomainObject cdo2;

        cdo.setIdentifierValue(IdentifierFactory.createUniqueIdentifier());
        cdo.setFloatValue(new Float(4.56));
        cdo.setLongValue(new Long(555));
        cdo.setStringValue("STRING");
        cdo.setByteArrayValue(new ByteArray(new byte[]{0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00}));
        cdo.setBooleanValue(new Boolean(false));
        cdo.setDateValue(new Date(System.currentTimeMillis()-10000));

        //update it with new values
        cdop.updateComprehensiveDomainObject(cdo);
        
        cdo2=cdop.getComprehensiveDomainObjectById(cdo.getId());
        
        assertEquals(cdo.getId(),cdo2.getId());
        assertEquals(cdo.getIdentifierValue(),cdo2.getIdentifierValue());
        assertEquals(cdo.getFloatValue(),cdo2.getFloatValue());
        assertEquals(cdo.getLongValue(),cdo2.getLongValue());
        assertEquals(cdo.getStringValue(),cdo2.getStringValue());
        assertEquals(cdo.getByteArrayValue(),cdo2.getByteArrayValue());
        assertEquals(cdo.getBooleanValue(),cdo2.getBooleanValue());
        
        //are we within a second (account for conversion imprecise-ness)
        assertTrue(Math.abs(cdo.getDateValue().getTime()-cdo2.getDateValue().getTime())<=1000);
        
        
    }
    
    public void testDelete(){
        
        ComprehensiveDomainObject cdo=new ComprehensiveDomainObjectImpl();
        
        ComprehensiveDomainObjectDAO cdop=(ComprehensiveDomainObjectDAO)DAOFactory.createDAOInstance(ComprehensiveDomainObject.class);
        
        cdop.addComprehensiveDomainObject(cdo);
        ComprehensiveDomainObject cdo2;

        
        cdo2=cdop.getComprehensiveDomainObjectById(cdo.getId());

        //should be there
        assertNotNull(cdo2);
        
        cdop.deleteComprehensiveDomainObject(cdo);
        
        //should be gone
        cdo2=cdop.getComprehensiveDomainObjectById(cdo.getId());
        assertNull(cdo2);
        
        
    }
    
/*    public void testLoad(){
        ComprehensiveDomainObject cdo=new ComprehensiveDomainObject();
        
        cdo.setIdentifierValue(IdentifierFactory.createUniqueIdentifier());
        cdo.setFloatValue(new Float(1.23));
        cdo.setLongValue(new Long(333));
        cdo.setStringValue("string");
        cdo.setByteArrayValue(new ByteArray(new byte[]{0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09}));
        cdo.setBooleanValue(new Boolean(true));
        cdo.setDateValue(new Date());

        ComprehensiveDomainObjectDAO cdop=(ComprehensiveDomainObjectDAO)DAOFactory.createDAOInstance(ComprehensiveDomainObject.class);

        long startTime;
        
        startTime=System.currentTimeMillis();
        
        for (int i=1;i<1000;i++)
            cdop.addComprehensiveDomainObject(cdo);
    
        long endTime;
        
        endTime=System.currentTimeMillis();
        
        long diffTime=endTime-startTime;
        
    }        
  */      
        
}