package com.iwombat.foundation.persist;

import junit.framework.*;

import java.util.*;
import com.iwombat.util.*;
import com.iwombat.foundation.*;
import com.iwombat.foundation.environment.*;


/**
 * Some simple tests.
 *
 */
public class StringIdentifierDomainObjectDAOTest extends TestCase {
    
    String defaultResultSetToBeanMapperClass;
    String defaultBeanToSqlParameterMapperClass;
    Properties persistenceOptions;

    private static final String CHAR_ID_BEAN_TO_SQL_PARAMETER_MAPPER_CLASS= "com.iwombat.foundation.persist.CharIdentifierBeanToSqlParameterMapper";
    private static final String CHAR_ID_RESULTSET_TO_BEAN_MAPPER_CLASS = "com.iwombat.foundation.persist.CharIdentifierResultSetToBeanMapper";
    
    private static final String IDENTIFIER_CLASSNAME="com.iwombat.foundation.Identifier";
    
    public StringIdentifierDomainObjectDAOTest(String name) {
        super(name);
    }
    
    public void setUp() {
        
        persistenceOptions=ConfigurationLocator.getProperties(PersistenceConstants.PERSISTENCE_OPTIONS_KEY);
        
        //save the current settings
        defaultBeanToSqlParameterMapperClass = persistenceOptions.getProperty(PersistenceConstants.BEAN_TO_SQL_PARAMETER_MAPPER_KEY);
        defaultResultSetToBeanMapperClass = persistenceOptions.getProperty(PersistenceConstants.RESULTSET_TO_BEAN_MAPPER_KEY);
        
        //set Identifier mapping to java.sql.Types.VARCHAR for this test
        persistenceOptions.setProperty(PersistenceConstants.BEAN_TO_SQL_PARAMETER_MAPPER_KEY, CHAR_ID_BEAN_TO_SQL_PARAMETER_MAPPER_CLASS);
        persistenceOptions.setProperty(PersistenceConstants.RESULTSET_TO_BEAN_MAPPER_KEY, CHAR_ID_RESULTSET_TO_BEAN_MAPPER_CLASS);
        
    }
    public void tearDown(){
        
        //restore it to what was saved
        persistenceOptions.setProperty(PersistenceConstants.BEAN_TO_SQL_PARAMETER_MAPPER_KEY, defaultBeanToSqlParameterMapperClass);
        persistenceOptions.setProperty(PersistenceConstants.RESULTSET_TO_BEAN_MAPPER_KEY, defaultResultSetToBeanMapperClass);
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite(StringIdentifierDomainObjectDAOTest.class);
        return suite;
    }
    
    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }

    public void testCreate(){
    
        StringIdentifierDomainObject cdo=new StringIdentifierDomainObjectImpl();
        
        cdo.setStringValue("string");

        StringIdentifierDomainObjectDAO cdop=(StringIdentifierDomainObjectDAO)DAOFactory.createDAOInstance(StringIdentifierDomainObject.class);

        
        cdop.addStringIdentifierDomainObject(cdo);
        StringIdentifierDomainObject cdo2;
        
        cdo2=cdop.getStringIdentifierDomainObjectById(cdo.getId());
        
        assertEquals(cdo.getId(),cdo2.getId());
        assertEquals(cdo.getStringValue(),cdo2.getStringValue());
        
    }
    
    public void testCreateNulls(){
        
        StringIdentifierDomainObject cdo=new StringIdentifierDomainObjectImpl();
        
        StringIdentifierDomainObjectDAO cdop=(StringIdentifierDomainObjectDAO)DAOFactory.createDAOInstance(StringIdentifierDomainObject.class);
        
        //cdo.setStringValue("blah");
        
        cdop.addStringIdentifierDomainObject(cdo);
        StringIdentifierDomainObject cdo2;
        
        cdo2=cdop.getStringIdentifierDomainObjectById(cdo.getId());

        assertEquals(cdo.getId(),cdo2.getId());
        assertNull(cdo2.getStringValue());
        
    }
    

    public void testUpdate(){
        
        StringIdentifierDomainObject cdo=new StringIdentifierDomainObjectImpl();
        
        cdo.setStringValue("string");

        StringIdentifierDomainObjectDAO cdop=(StringIdentifierDomainObjectDAO)DAOFactory.createDAOInstance(StringIdentifierDomainObject.class);
        
        cdop.addStringIdentifierDomainObject(cdo);
        StringIdentifierDomainObject cdo2;

        cdo.setStringValue("STRING");

        //update it with new values
        cdop.updateStringIdentifierDomainObject(cdo);
        
        cdo2=cdop.getStringIdentifierDomainObjectById(cdo.getId());
        
        assertEquals(cdo.getId(),cdo2.getId());
        assertEquals(cdo.getStringValue(),cdo2.getStringValue());
        
    }
    
    public void testDelete(){
        
        StringIdentifierDomainObject cdo=new StringIdentifierDomainObjectImpl();
        
        StringIdentifierDomainObjectDAO cdop=(StringIdentifierDomainObjectDAO)DAOFactory.createDAOInstance(StringIdentifierDomainObject.class);
        
        cdop.addStringIdentifierDomainObject(cdo);
        StringIdentifierDomainObject cdo2;

        
        cdo2=cdop.getStringIdentifierDomainObjectById(cdo.getId());

        //should be there
        assertNotNull(cdo2);
        
        cdop.deleteStringIdentifierDomainObject(cdo);
        
        //should be gone
        cdo2=cdop.getStringIdentifierDomainObjectById(cdo.getId());
        assertNull(cdo2);
        
        
    }
    
/*    public void testLoad(){
        StringIdentifierDomainObject cdo=new StringIdentifierDomainObject();
        
        cdo.setIdentifierValue(IdentifierFactory.createUniqueIdentifier());
        cdo.setFloatValue(new Float(1.23));
        cdo.setLongValue(new Long(333));
        cdo.setStringValue("string");
        cdo.setByteArrayValue(new ByteArray(new byte[]{0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09}));
        cdo.setBooleanValue(new Boolean(true));
        cdo.setDateValue(new Date());

        StringIdentifierDomainObjectDAO cdop=(StringIdentifierDomainObjectDAO)DAOFactory.createDAOInstance(StringIdentifierDomainObject.class);

        long startTime;
        
        startTime=System.currentTimeMillis();
        
        for (int i=1;i<1000;i++)
            cdop.addStringIdentifierDomainObject(cdo);
    
        long endTime;
        
        endTime=System.currentTimeMillis();
        
        long diffTime=endTime-startTime;
        
    }        
  */      
        
}