// (c) Copyright 2001-2007 iwombat.com all rights reserved

package com.iwombat.foundation.uuid;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;


/**
 * @author bobc
 *
 */
public class UUIDTest extends TestCase {

    /**
     * @param args test args
     */
    public static void main(String[] args) {
    }

    /**
     * @param arg0 test args
     */
    public UUIDTest(String arg0) {
        super(arg0);
    }


    /**
     * Test out the equals method of objectid
     */
    public void testEquals() {
            UUID idOne = new UUID("00112233445566778899aabbccddeeff");
            UUID idTwo = new UUID("00112233445566778899aabbccddeeee");
            assertTrue("UUIDs is not equal to itself", idOne.equals(idOne));
            assertTrue("Two UUIDs are equal", !idOne.equals(idTwo));
            
    }
    
    /** 
     * Test exporting an id as a hex string and recreating
     */
    public void testStringRecreate() {
        UUID idOne = new UUID("00112233445566778899aabbccddeeff");
        String idString = idOne.toHexString();
        UUID idOneAgain = new UUID(idString);
        assertTrue("Remanufactued UUIDs is not equal to itself", idOne.equals(idOneAgain));
    }
    
    /**
     * Tests the format of UUID Strings. 
     */
    public void testToString() {
        UUID id = UUIDFactory.createUUID();
        assertEquals(32, id.toString().length());
        assertEquals(id.toHexString(), id.toString());
        assertTrue(id.toHexString().matches("[0-9a-f]+"));
    }
    
    /**
     * test hashcode
     *
     */
    public void testHashCode() {
//      check that equal ObjectIds have equal hashes
        UUID id = UUIDFactory.createUUID();
        UUID idCopied = new UUID(id.toHexString());
        assertTrue("Equal UUIDs do not have equal hashes", 
             (id.hashCode() == idCopied.hashCode()));
    }
    /**
     * Create a pile of ObjectIds and make sure they are all unique
     */
    public void testUnique() {

        int listLength = 100;
        ArrayList uuidList = new ArrayList();
        
        // create the test data
        for (int i = 0; i < listLength; i++) {
            uuidList.add(UUIDFactory.createUUID());
        }

        uniqueListChecker(uuidList);
        
    }

    private void uniqueListChecker(List uuidList) {
        Iterator iter = uuidList.iterator();
        while (iter.hasNext()) {
            UUID listId = (UUID) iter.next();
        
            Iterator iter2 = uuidList.listIterator(uuidList.indexOf(listId) + 1);
            while (iter2.hasNext()) {
                UUID otherListId = (UUID) iter2.next();
           
                assertTrue("Different UUIDs equal", 
                            !listId.equals(otherListId));
                
                // not an actual requirement for hashCodes but something to strive for
                assertTrue("Different UUIDs have same hash", 
                            !(listId.hashCode() == otherListId.hashCode()));
            }
        }
    }
    
    /**
     * Tests how UUIDs act when passed invalid arguments.
     */
    public void testIllegalArguments() {
        // id with 15 octets.
        try {
            new UUID("00112233445566778899aabbccddee");
            fail("Didn't throw exception for improper length with String constructor");
        } catch (IllegalArgumentException e) {
            // do nothing
        }
        
        try {
            UUID idOne = new UUID("00112233445566778899aabbccddeeff");
            byte[] bytes = idOne.getValue();
            byte[] shortId = new byte[15];
            for (int i = 0; i < 15; i++) {
                shortId[i] = bytes[i];
            }
            new UUID(shortId);
            fail("Didn't throw exception for improper length with byte[] constructor");
        } catch (IllegalArgumentException e) {
            // do nothing
        }
    }
    
    
    private static final int NUM_THREADS = 10;
    /**
     * Test creation of UUIDs on multiple threads
     */
    public void testThreadedUnique() {
       
        ArrayList threads = new ArrayList();
        List results = Collections.synchronizedList(new ArrayList());
        
        
        for (int i = 0; i < NUM_THREADS; i++) {
            threads.add(new UUIDThreadHelper(results));
        }
        
        
        Iterator iter = threads.iterator();
        while (iter.hasNext()) {
            Thread helper = (Thread)iter.next();
            helper.start();
        }
        
        int counter = 0;
        while (UUIDThreadHelper.getFinishedCount() < NUM_THREADS) {
            counter++;           
        }
        
        // ystem.out.println("Done processing: resultCount=" + results.size());
        this.uniqueListChecker(results);
        
    }

}
