// (c) 2001-2002 iWombat.com All rights reserved
               

package com.iwombat.util;

import junit.framework.*;
import java.util.*;

/**
 * Tests some basic functionality of VariantList
 *
 * @author  $Author: bob $
 * @version $Revision: 1.4 $
 * @date    $Date: 2002/03/14 05:30:49 $
 */
                           
public class VariantListTest extends TestCase {
    
    public VariantListTest(java.lang.String testName) {
        super(testName);
    }        
    
    
    public static Test suite() {
        TestSuite suite = new TestSuite(VariantListTest.class);
        return suite;
    }

    public void testConstructor(){
        
        VariantList vl=new VariantList();
        
        assertTrue("Could not create new VariantList with empty constructor",vl!=null);
        
        ArrayList a=new ArrayList();
        
        a.add(new Variant(1));
        a.add(new Variant(2));
        a.add(new Variant(3));
        
        vl=new VariantList(a);
        
        assertTrue("Could not create new VariantList with Variant Collection as constructor",
        
        (vl.getItem(0).getint()==1 &&
        vl.getItem(1).getint()==2 &&
        vl.getItem(2).getint()==3)
        
        );
        
    }

    public void testAdd(){
    
        VariantList vl=new VariantList();
        Variant v=new Variant();
        
        vl.add(v);
        
        assertTrue("could not retrieve added Variant",v==vl.getItem(0));
        
        vl=new VariantList();
        
        vl.add(new Variant(1));
        vl.add(new Variant(2));
        
        vl.add(1,new Variant("x"));
        
        assertTrue("could not retrieve Variant added at a specified position",vl.getItem(1).getString().equals("x"));
    }

    
    public void testAddAll(){
    
        VariantList vl= new VariantList();
        
        vl.add(new Variant(0));
        vl.add(new Variant(1));
        vl.add(new Variant(2));
        vl.add(new Variant(3));
        vl.add(new Variant(4));
        vl.add(new Variant(5));
        vl.add(new Variant(6));
        vl.add(new Variant(7));
        vl.add(new Variant(8));
        vl.add(new Variant(9));

        VariantList vl2=new VariantList();
        
        ArrayList loader=new ArrayList();
        
        for (Iterator i=vl.iterator();i.hasNext();)
            loader.add(i.next());
        
        
        vl2.addAll(loader);
        
        int index=0;
        for (Iterator i=vl2.iterator();i.hasNext();)
            assertTrue("addAll failed- new VariantList does not match original VariantList",i.next()==vl.get(index++));
        
        vl2=new VariantList();
        
        
        vl2.add(new Variant(10));
        vl2.add(new Variant(11));
        vl2.addAll(2,loader);
        
        index=2;
        for (Iterator i=vl.iterator();i.hasNext();)
            assertTrue("addAll failed- new VariantList does not match original subset of VariantList",i.next()==vl2.get(index++));
    }
    
    public void testContains(){
    
        Variant v1=new Variant("a");
        Variant v2=new Variant("b");
        Variant v3=new Variant("c");
        
        VariantList vl=new VariantList();
        
        vl.add(v1);
        vl.add(v2);
        vl.add(v3);
        
        assertTrue("testContains() returned false even though tested Variant was added to the list",vl.contains(v2));
        
        vl.remove(v2);
        assertTrue("testContains() returned true after removing tested Variant from list", !vl.contains(v2));
        
    }
    
    public void testGetItem(){
    
        Variant v1=new Variant("a");
        Variant v2=new Variant("b");
        Variant v3=new Variant("c");
        
        Variant testVariant;
        
        VariantList vl=new VariantList();
        
        
        vl.add(v1);
        vl.add(v2);
        vl.add(v3);
        
        testVariant=vl.getItem(1);
        
        assertTrue("could not retrieve item from VariantList",testVariant==v2);
        
        
        
    }
    
    public void testIndexOf(){
        
        Variant v1=new Variant("a");
        Variant v2=new Variant("b");
        Variant v3=new Variant("c");
        
        VariantList vl=new VariantList();
        
        
        vl.add(v1);
        vl.add(v2);
        vl.add(v3);
        
        assertTrue ("could not get the proper index for a Variant that was added",vl.indexOf(v2)==1);
    }
    
    public void testLastIndexOf(){
        
        Variant v1=new Variant("a");
        Variant v2=new Variant("b");
        Variant v3=new Variant("c");
        
        VariantList vl=new VariantList();
        
        
        vl.add(v1);
        vl.add(v2);
        vl.add(v3);
        vl.add(v2);
        
        assertTrue("could not get the proper last index for a variant that was added",vl.lastIndexOf(v2)==3);
    }
    
    public void testRemoveItem(){
        
        VariantList vl=new VariantList();
        Variant v=new Variant();
        Variant test=new Variant();
        vl.add(v);
        vl.remove(0);
        
        assertTrue("could not remove Variant by index",vl.size()==0);
        
        vl.add(v);
        vl.remove(v);
        assertTrue("could not remove Variant by object reference",vl.size()==0);
        
        
        vl.add(v);
        
        test=vl.removeItem(0);
        
        assertTrue("could not retrieve removed variant by index",test==v && vl.size()==0);
    }

    public void testSet(){
     
        Variant v1=new Variant("a");
        Variant v2=new Variant("b");
        Variant v3=new Variant("c");
        Variant v4=new Variant("d");
        
        VariantList vl=new VariantList();
        
        vl.add(v1);
        vl.add(v2);
        vl.add(v3);
        vl.add(v4);

        vl.set(1,v4);
        
        assertTrue("could not retrieve correct Variant set at specific position within VariantList",vl.getItem(1)==v4);
    }
    
    public void testIllegalArguments(){
    
        ArrayList illegalList=new ArrayList();
        VariantList vl;
        
        illegalList.add(new Variant("ok"));
        illegalList.add(new String("ack!"));
        illegalList.add(new Variant("no prob"));
        
        try{
        
            vl=new VariantList(illegalList);
            assertTrue("VariantList constructor that loads from a collection failed to block non-Variant collection item",false);
            
        }
        catch (IllegalArgumentException e){
        }
        
        vl=new VariantList();

        try{
            vl.addAll(illegalList);
            assertTrue("VariantList addAll() failed to block non-Variant collection item",false);
        }
        catch (IllegalArgumentException e){
        }
        
        vl.add(new Variant("jello"));
        
        try{
         
            vl.addAll(0,illegalList);
            assertTrue("VariantList addAll() failed to block non-Variant collection item when adding at a specified index",false);
        }
        catch (IllegalArgumentException e){
        }
        
    }
    
    public void testUnsupportedOperations(){
     
        String illegalObject=new String("trouble maker");

        
        VariantList vl=new VariantList();
        
        try{
            vl.add(illegalObject);
            assertTrue("VariantList failed to block non Variant instance from being added",false);
        }
        catch (UnsupportedOperationException e){
        }

        /////////////////////////////
        
        vl=new VariantList();
        vl.add(new Variant("good object"));
        
        try{
            vl.add(0,illegalObject);
            assertTrue("VariantList failed to block non Variant instance from being added at specific index",false);
        }
        catch (UnsupportedOperationException e){
        }


        /////////////////////////////
        
        vl=new VariantList();
        vl.add(new Variant("good object"));
        
        try{
            vl.add(illegalObject);
            assertTrue("VariantList failed to block non Variant instance from being added",false);
        }
        catch (UnsupportedOperationException e){
        }
        
        /////////////////////////////
        
        vl=new VariantList();
        vl.add(new Variant("good object"));
        
        try{
            vl.set(0,illegalObject);
            assertTrue("VariantList failed to block non Variant instance from being added",false);
        }
        catch (UnsupportedOperationException e){
        }
    }
    

}
